'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getImports = undefined;

var _nodeDir = require('node-dir');

var _nodeDir2 = _interopRequireDefault(_nodeDir);

var _shiftParser = require('shift-parser');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// https://stackoverflow.com/a/45462325
/**
 *  Copyright (c) 2017, Marcelo Jorge Vieira <metal@alucinados.com>
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Affero General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Affero General Public License for more details.
 *
 *   You should have received a copy of the GNU Affero General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

var sortObject = function sortObject(obj) {
  return Object.keys(obj).sort().reduce(function (a, v) {
    a[v] = obj[v];
    return a;
  }, {});
};

var _internalImports = function _internalImports(groupImportsArray, module) {
  for (var j = 0; j < module.length; j++) {
    var moduleSpecifier = module[j].moduleSpecifier;
    var namedImports = module[j].namedImports;
    var defaultBinding = module[j].defaultBinding;
    var arrImports = groupImportsArray[moduleSpecifier] || (groupImportsArray[moduleSpecifier] = []);
    for (var k = 0; k < namedImports.length; k++) {
      var item = namedImports[k].binding.name;
      if (arrImports.indexOf(item) === -1) {
        arrImports.push(item);
      }
    }
    if (defaultBinding && arrImports.indexOf(defaultBinding.name) === -1) {
      arrImports.push(defaultBinding.name);
    }
    arrImports.sort();
  }
};

var _groupImports = function _groupImports(importsList) {
  var groupImportsArray = {};
  for (var i = 0; i < importsList.length; i++) {
    _internalImports(groupImportsArray, importsList[i].items);
  }
  return sortObject(groupImportsArray);
};

var defaultSettings = {
  match: /.js$/,
  exclude: [/^\./],
  excludeDir: ['dist', 'lib', 'node_modules']
};

var _parseImports = function _parseImports(content) {
  // Remove comments
  // http://upshots.org/javascript/javascript-regexp-to-remove-comments
  content = content.replace(/\/\*[\s\S]*?\*\/|([^\\:]|^)\/\/.*$/gm, '');

  // https://gist.github.com/manekinekko/7e58a17bc62a9be47172
  var regex = /import(?:["'\s]*([\w*{}\n, ]+)from\s*)?["'\s].*([@\w/_-]+)["'\s].*;?/g;
  var matches = [];
  var match = regex.exec(content);
  while (match != null) {
    matches.push(match[0]);
    match = regex.exec(content);
  }
  if (matches.length > 0) {
    var moduleAST = (0, _shiftParser.parseModule)(matches.join('\n'));
    return moduleAST;
  }
  return [];
};

var getImports = exports.getImports = function getImports(dirName, options, callback) {
  if (typeof options === 'function') {
    callback = options;
    options = null;
  }
  var importsList = [];
  var settings = options || defaultSettings;
  return _nodeDir2.default.readFiles(dirName, settings, function (err, content, next) {
    if (err) throw err;
    var result = _parseImports(content);
    importsList = importsList.concat(result);
    next();
  }, function (err, files) {
    if (err) throw err;
    callback(_groupImports(importsList), files);
  });
};